# 
# If "?????????" then File - Reopen with Encoding... windows-1251
#
#   .. ,  
#     
# .. , ..
#  4    "    R"
# http://www.landscape.edu.ru/edu_help4_R.shtml
# 
#  "      
#  -     " 
# 
#   -     - 
# 
#  
# 1.      R
# 2.   
# 3.  -  ()
#  3.1.   
#  3.2.   
#  3.3.   
# 4.   
# 5.       

#      R
##########################################################################################
#  1.1:   
##########################################################################################
# 
#           
# www.landscape.edu.ru/files/DSM_METdataset_v0.99.zip 
#  https://yadi.sk/i/xEJB1-9DyeqUC
#
#    R - ...
#
##########################################################################################
#  1.2:      R
##########################################################################################

library(raster)
library(car)
library(leaflet)
library(rasterVis)
library(randomForest) 
library(ithir)
library(ellipse)
library(MASS)
library(relaimpo)
library(e1071)

#     
install.packages('car', dependencies = TRUE)
install.packages('leaflet', dependencies = TRUE)
install.packages('raster', dependencies = TRUE)
install.packages('ellipse', dependencies = TRUE)
install.packages('randomForest', dependencies = TRUE)
install.packages("relaimpo", dependencies = TRUE)
install.packages("rasterVis", dependencies = TRUE)
install.packages("relaimpo", dependencies = TRUE)
install.packages("MASS", dependencies = TRUE)
install.packages("e1071", dependencies = TRUE)

install.packages("devtools") #     "ithir"    
library(devtools)
install_bitbucket("brendo1001/ithir/pkg", dependencies = TRUE)
#
##########################################################################################
#  1.3:    
##########################################################################################
#
setwd('C:/Dropbox/0_EDU/R/METsid') #    
setwd('D:/Dropbox/0_EDU/R/METsid') #    
setwd("C:/Users/User/YandexDisk/R/MET")

#    SAGA-grid    
covStack = stack(list.files(path = "...", pattern='*.sdat'))
proj4string(covStack) = CRS('+init=epsg:32637') # (UTM (WGS84),  37N)
#    
names(covStack) = c("z","slp","cdep","facc","twi","tpi10","tpi60","plncurv","prfcurv","gnrcurv", "cscurv","aacn", "simwe") 

par(mar=c(2, 2, 1.5, 0.2))  #      , 
plot(covStack[[1]],  main = ". , ", col=terrain.colors(255))
plot(covStack[[2]],  main = ", ", col=grey(seq(0,1,len = 25)))
plot(covStack[[3]],  main = "  , ", col=colorRampPalette(c("lightgrey", "red"))( 10) )
plot(covStack[[4]],  main = " , 2", col=colorRampPalette(c("lightgrey", "darkblue"))( 50))
plot(covStack[[5]],  main = " , log(CA/slope)")
plot(covStack[[6]],  main = ".    10 , ", col=colorRampPalette(c("dodgerblue4", "lightgrey", "darkorange3")) (50), zlim = c(-0.3,0.3))
plot(covStack[[7]],  main = ".    60 , ", col=colorRampPalette(c("dodgerblue4", "lightgrey", "darkorange3")) (50), zlim = c(-1.0,1.0))
plot(covStack[[8]],  main = " ", col=colorRampPalette(c("dodgerblue4", "lightgrey", "darkorange3")) (50), zlim = c(-1.0,1.0))
plot(covStack[[9]],  main = " ", col=colorRampPalette(c("darkblue", "lightgrey", "red"))( 50))
plot(covStack[[10]], main = " ", col=colorRampPalette(c("darkblue", "lightgrey", "red"))( 50))
plot(covStack[[11]], main = "- ", col=colorRampPalette(c("darkblue", "lightgrey", "red"))( 50))
plot(covStack[[12]], main = "  , ", col=colorRampPalette(c("lightgrey", "red"))( 30))
plot(covStack[[13]], main = "  , ", col=colorRampPalette(c("lightgrey", "darkblue"))( 50))

#     (  )
contour(covStack[[1]], drawlabels = TRUE, axes = FALSE, frame = TRUE, add=TRUE, col="black")

#      
DSM_data = read.csv2("met_soilpoints.txt", header = TRUE, sep = "\t", 
                     quote = "\"", dec = ".", stringsAsFactors = FALSE) 
str(DSM_data) #   
DSM_data$sid = factor(DSM_data$sid,ordered=TRUE, levels=c("","","","")) #     
table(DSM_data$sid) #   (  )

#     -   Xm, Ym 
coordinates(DSM_data) = ~ Xm + Ym #   "raster"
proj4string(DSM_data) = CRS('+init=epsg:32637') #    (UTM (WGS84),  37N)

#   (  )    
points(DSM_data, pch = 21:21, cex=0.5, col="darkslateblue") 
#    
text(DSM_data$Xm, DSM_data$Ym, labels=DSM_data$sid, cex = 0.7, pos = 3, offset = 0.5)

#  Leaflet map widget      (   )
leaflet() %>% 
  addMarkers(lng = 36.09170, lat = 51.57200, popup = "") %>% 
  addProviderTiles("Esri.WorldImagery") 

leaflet() %>% 
  addTiles() %>% 
  addProviderTiles("Esri.WorldImagery") %>% 
  addCircleMarkers(data = spTransform(DSM_data, CRS('+init=epsg:4326')), 
                  popup = paste0("<strong>sid: </strong>", DSM_data$sid), radius = 2, color = "red")

#      (     covStack)
covRST = covStack[[6]] # [6] -     10 
covRST = projectRaster(covRST, crs = '+init=epsg:4326') #     / (WGS84)
pal = colorNumeric(palette=rev(brewer.pal(3,"RdBu")), domain = values(covRST), na.color = "transparent") #  
leaflet() %>% 
  addProviderTiles("Esri.WorldImagery") %>% 
  addRasterImage(covRST, colors = pal, opacity = 0.99) %>%
  addLegend(pal = pal, values = values(covRST), title = "topographic wetness index") %>%
  addCircleMarkers(data = spTransform(DSM_data, CRS('+init=epsg:4326')), popup = paste0("<strong>sid: </strong>", DSM_data$sid), radius = 2, color = "black")
#     ,     


################################################################################################################
# 2.    (   )
################################################################################################################
#
##########################################################################################
#  2.1:       
##########################################################################################
#
DSM_data = data.frame(extract(covStack, DSM_data, method="simple", sp=1))
DSM_data = data.frame(DSM_data[ ,c("sid","z","slp","cdep","facc","twi","tpi10", "tpi60","plncurv","prfcurv","gnrcurv","cscurv","aacn", "simwe")])
format(head(DSM_data), digits=1, scientific=FALSE) #  6 
#
##########################################################################################
#  2.2:   
##########################################################################################
#
par(mar=c(0, 0, 0, 0))  #      , 
round(cor(DSM_data[ ,-1]), 2)   #   
symnum(cor(DSM_data[ ,-1]))  #    
plotcorr(cor(DSM_data[ ,-1]), type="lower")#    
#
##########################################################################################
#  2.3:      
##########################################################################################
#
#         library(car)
scatterplotMatrix(~ slp + simwe + log(facc) + twi|sid, data=DSM_data, cex=0.8) 
scatterplotMatrix(~ tpi10 + twi + log(facc)|sid, data=DSM_data, cex=0.8)
#      
scatterplotMatrix(~ twi + log(facc*(tpi10+1)) + log(facc)| sid, data=DSM_data, cex=0.8)

#    
lmMod = lm(as.numeric(sid) ~ ., data = DSM_data)  #   ,      
relImportance = calc.relimp(lmMod, type = "lmg", rela = TRUE)  #       100
sort(round(relImportance$lmg, 2), decreasing=TRUE)  #    (relative importance)

#    
lmMod = lm(as.numeric(sid) ~ twi + tpi10 + cdep + cscurv + aacn, data = DSM_data)  #   
relImportance = calc.relimp(lmMod, type = "lmg", rela = TRUE)  #       100
sort(round(relImportance$lmg, 2), decreasing=TRUE)  #    (relative importance)

#   (    )
#         ,     
par(mar=c(4, 4, 1, 0.2))  #      , 
#   (HEX-  "", "", "", "")
sid_col = c("#92DC99", "#D39829", "#B7621C", "#793405") 
palette(sid_col)
#      
plot(as.vector(covStack$tpi10), as.vector(covStack$twi), col="grey", pch=20, cex=0.6, 
     xlim = c(-0.2,0.2), ylim = c(4,16),  xlab=".    10 , ", ylab=" ") 
#  ,    
points(DSM_data$tpi10, DSM_data$twi, col=DSM_data$sid, lwd=2, cex=1.2) 
#       50  90%
dataEllipse(DSM_data$tpi10, DSM_data$twi, DSM_data$sid, c(0.5,0.95), add=TRUE, 
            plot.points=TRUE, lwd=1, fill=TRUE, fill.alpha=0.15, pch=c(21,21,21,21), col=sid_col)
#  
legend("topright",bg="white",pch=c(21,21,21,21), legend=c("","","", ""), 
       inset = .02, bty="n", col= sid_col, cex=1.2, pt.lwd=2, pt.bg="white")

# :       

####################################################################################################
# 3.  -  ()
####################################################################################################
#
##########################################################################################
#  3.1:   
##########################################################################################
#     2 
# 70% -    ; 30% -  (  )
set.seed(123) #    
training <- sample(nrow(DSM_data), 0.70 * nrow(DSM_data)) #  70%  
training #     
DSM.train = DSM_data[training, ]  #  
DSM.verif = DSM_data[-training, ] #   
#
##########################################################################################
#  3.2:    (Linear Discriminant Analysis (lda)
##########################################################################################
# 
#    ( MASS) 
mod.lda = lda(sid ~ tpi10 + twi, data=DSM_data)  #      
mod.lda #  

#      
mod.lda = lda(sid ~ tpi10 + twi, data=DSM_data, prior=c(0.15,0.30,0.40,0.15)) 
mod.lda

#       
pred.lda = as.data.frame(predict(mod.lda)) # 
format(head(pred.lda), digits=1, scientific=FALSE) #   , ,  

#   :   ( - ,  - ),  (ithir)
goofcat(observed = DSM_data$sid, predicted = pred.lda$class)  

#      () 
map.lda.c = predict(covStack, mod.lda) #      
#       
map.lda.p1 = predict(covStack, mod.lda, type = "probs", index=2) # 
map.lda.p2 = predict(covStack, mod.lda, type = "probs", index=3) # 
map.lda.p3 = predict(covStack, mod.lda, type = "probs", index=4) # 
map.lda.p4 = predict(covStack, mod.lda, type = "probs", index=5) # 
map.lda.p4 = predict(covStack, mod.lda, type = "probs", index=5) # 
#     
map.lda.p   = stack(map.lda.p1, map.lda.p2, map.lda.p3, map.lda.p4) 
names (map.lda.p) = c('', '', '', '') # 

#      ( 'car')
par(mar=c(4, 4, 1, 0.2))  #      , 
#   (HEX-    "", "", "", "")
sid_col = c("#5AA75F", "#B77E29", "#A25817", "#793405") #  
palette(c("#79C17E", "#E0B462", "#D89156", "#934D08"))  #  
#      
plot(as.vector(covStack$tpi10), as.vector(covStack$twi), col=as.vector(map.lda.c), pch=20, cex=0.6, xlim = c(-0.2,0.2),
     ylim = c(4,16),  xlab=".    10 , ", ylab=" ") 
#       50  90%
dataEllipse(DSM_data$tpi10, DSM_data$twi, DSM_data$sid, c(0.5,0.95), add=TRUE, plot.points=TRUE, lwd=1,
            fill=TRUE, fill.alpha=0.15, pch=c(21,22,24,25), col=sid_col)
#  
legend("topright",bg="white", legend=c("","","", ""), inset = .02, 
       bty="n", col= sid_col, pch=c(21,22,24,25), cex=1.2, pt.lwd=2, pt.bg="white")

#      ( )
cat = levels(as.factor(map.lda.c))[[1]]    #      
cat[["sid"]] = c("","", "","")  #   ()
levels(map.lda.c) = cat      #      ( )
sid_col = c("#92DC99", "#E6BE75", "#D89156", "#934D08") #  HEX     
levelplot(map.lda.c, col.regions=sid_col, xlab="", ylab="") #   ( rasterVis)

#    
par(mfrow=c(1,4)) #   4    ,  - dev.off() 
par(mar=c(2.5, 0, 1, 2))  #      , 
for (i in 1:4) plot(map.lda.p[[i]], col = gray.colors(100, start = 0.9, end = 0.1))  

#      
map.lda.pmax = stackApply(map.lda.p, indices=c(1,1,1,1), fun=max) 
cellStats(map.lda.pmax, mean) #   
par(mfrow=c(1,1)) #        
plot(map.lda.pmax, col = grey(seq(0, 1, len = 25)))

#      (70%)
mod.lda.trn = lda(sid ~ tpi10 + twi, data=DSM.train) 
#   
pred.lda.trn = as.data.frame(predict(mod.lda.trn)) # 70%
pred.lda.ver = as.data.frame(predict(mod.lda.trn, DSM.verif)) # 30%
#      
goofcat(observed = DSM.train$sid, predicted = pred.lda.trn$class) # 70% 
goofcat(observed = DSM.verif$sid, predicted = pred.lda.ver$class) # 30%
#
##########################################################################################
#  3.3:       
##########################################################################################
# 
#     
mod.rf  = randomForest(sid~tpi10 + twi, data=DSM_data) 
#      
mod.rf  = randomForest(sid~tpi10 + twi, data=DSM_data, ntree=300, mtry=1, sampsize=50, nodesize=5, replace=TRUE) 

#      
plot(mod.rf, col=c("black", "#92DC99", "#E6BE75", "#D89156", "#934D08"))

print(mod.rf) #   
importance(mod.rf) #    
varImpPlot(mod.rf) #     

#           
pred.rf = as.data.frame(predict(mod.rf)) #   
head(pred.rf)

#   
goofcat(observed = DSM_data$sid, predicted = pred.rf) 

#        ()
map.rf.c = predict(covStack, mod.rf) #   
map.rf.p1 = predict(covStack, mod.rf, type = "prob", index=1) # 
map.rf.p2 = predict(covStack, mod.rf, type = "prob", index=2) # 
map.rf.p3 = predict(covStack, mod.rf, type = "prob", index=3) # 
map.rf.p4 = predict(covStack, mod.rf, type = "prob", index=4) # 
#     
map.rf.p  = stack(map.rf.p1, map.rf.p2, map.rf.p3, map.rf.p4) 
names (map.rf.p) = c('', '', '', '') # 
par(mfrow=c(1,4)) #       
par(mar=c(2.5, 0, 1, 2))  #      , 
for (i in 1:4) plot(map.rf.p[[i]], col = gray.colors(100, start = 0.9, end = 0.1))
#      lda  rf 
for (i in 1:4) plot(map.lda.p[[i]], map.rf.p[[i]])

#      
par(mfrow=c(1,1)) #        
map.rf.pmax = stackApply(map.rf.p, indices=c(1,1,1,1), fun=max) 
cellStats(map.rf.pmax, mean) #   
plot(map.rf.pmax, col = grey(seq(0, 1, len = 25)))

#      
par(mfrow=c(1,1), mar=c(4, 4, 1, 0.2)) #             , 
#   (HEX-    "", "", "", "")
sid_col = c("#5AA75F", "#B77E29", "#A25817", "#793405") #  
palette(c("#79C17E", "#E0B462", "#D89156", "#934D08")) #  
plot(as.vector(covStack$tpi10), as.vector(covStack$twi), col=as.vector(map.rf.c), pch=20, cex=0.6, xlim = c(-0.2,0.2), ylim = c(4,16), 
     xlab=".    10 , ", ylab=" ")
dataEllipse(DSM_data$tpi10, DSM_data$twi, DSM_data$sid, c(0.5, 0.95), add = TRUE, plot.points = TRUE, lwd=1, 
            fill=TRUE, fill.alpha=0.15, pch=c(21,22,24,25), col=sid_col)
legend("topright",bg="white",pch=c(21,22,24,25), legend=c("","","", ""), inset = .02, bty="n",
       col=sid_col,pt.bg=c("white","white","white","white"))

#  
cat <- levels(as.factor(map.rf.c))[[1]] #      
cat[["sid"]] <- c("","", "","")  #   ()
levels(map.rf.c) <- cat                 #     
area_colors <- c("#92DC99", "#E6BE75", "#D89156", "#934D08")      #  HEX     
levelplot(map.rf.c, col.regions=area_colors, xlab="", ylab="")  #  

#      (70%)
mod.rf.trn = randomForest(sid~tpi10+twi, data=DSM.train, ntree=100,mtry=2) 
#   
pred.rf.trn = as.data.frame(predict(mod.rf.trn)) # 70%
pred.rf.ver = as.data.frame(predict(mod.rf.trn, DSM.verif)) # 30%
#      
goofcat(observed = DSM.train$sid, predicted = pred.rf.trn$predict) # 70% 
goofcat(observed = DSM.verif$sid, predicted = pred.rf.ver$predict) # 30%
#
##########################################################################################
#  3.4:      (Support Vector Machine  SVM) 
##########################################################################################
#
#     ( ) 
mod.svm = svm(sid ~ tpi10 + twi, data = DSM_data) #  1071
summary(mod.svm)
goofcat(observed=DSM_data$sid,predicted=predict(mod.svm,newdata=DSM_data)) #    

#   
svmtunepar = tune.svm(sid ~ tpi10 + twi, data = DSM_data, gamma = seq(0.25,2,0.25), cost = seq(1,501,25)) 
svmtunepar = tune.svm(sid ~ tpi10 + twi, data = DSM_data, gamma = seq(0.25,2,0.25), cost = seq(1,51,5), random=FALSE) 
summary(svmtunepar)
plot (svmtunepar,  xlab="gamma", ylab="cost")
svmtunepar$ # $best.parameters

#      
costs = table(DSM_data$sid) 
costs['']   = 10.20
costs['']   = 1.25 
costs['']   = 1.35
costs['']  = 1.20 
costs #   
mod.svm = svm(sid ~ tpi10 + twi,data = DSM_data, class.weights=costs, cost = 6, gamma=1) 
summary(mod.svm)
mod.svm
#    
goofcat(observed=DSM_data$sid,predicted=predict(mod.svm,newdata=DSM_data))

#    ( )
map.svm.c=predict(covStack, mod.svm, type = "class") 

#     
par(mfrow=c(1,1), mar=c(4, 4, 1, 0.2)) #             , 
#   (HEX-    "", "", "", "")
sid_col = c("#5AA75F", "#B77E29", "#A25817", "#793405") #  
palette(c("#79C17E", "#E0B462", "#D89156", "#934D08"))  #  
plot(as.vector(covStack$tpi10), as.vector(covStack$twi), col=as.vector(map.svm.c), pch=20, cex=0.6, xlim = c(-0.2,0.2), ylim = c(4,16), 
     xlab=".    10 , ", ylab=" ")
dataEllipse(DSM_data$tpi10, DSM_data$twi, DSM_data$sid, c(0.5, 0.95), add = TRUE, plot.points = TRUE, lwd=1, fill=TRUE, fill.alpha=0.15,
            pch=c(21,22,24,25), col=c("#92DC99", "#D39829", "#B7621C", "#793405"))
legend("topright",bg="white",pch=c(21,22,24,25), legend=c("","","", ""), inset = .02, bty="n",
       col=c("#92DC99", "#E6BE75", "#B7621C", "#793405"),pt.bg=c("white","white","white","white"))

#  ,    
cat = levels(as.factor(map.svm.c))[[1]] #      
cat[["sid"]] = c("","", "","") #   ()
levels(map.svm.c) = cat                 #     
area_colors = c("#92DC99", "#E6BE75", "#D89156", "#934D08")      #  HEX     
levelplot(map.svm.c, col.regions=area_colors, xlab="", ylab="")  #  

#      (70%)
mod.svm.trn = randomForest(sid~tpi10+twi, data=DSM.train, ntree=100,mtry=2) 
#   
pred.svm.trn = as.data.frame(predict(mod.svm.trn)) # 70%
pred.svm.ver = as.data.frame(predict(mod.svm.trn, DSM.verif)) # 30%
#      
goofcat(observed = DSM.train$sid, predicted = pred.svm.trn$predict) # 70% 
goofcat(observed = DSM.verif$sid, predicted = pred.svm.ver$predict) # 30%

##########################################################################################
#  4:   
##########################################################################################
#    
# 1)           (.tiff, .grd, .sdat  .: . writeRaster {raster})
map.lda.c = predict(covStack, mod.lda, type = "class", filename = "map_lda_c.tiff", 
                    format = "GTiff", overwrite = TRUE, datatype = "FLT4S") 
# 2)     
writeRaster(map.lda.c, filename = "map_lda_.tiff", 
            format = "GTiff", overwrite = TRUE, datatype = "FLT4S")

############################################################################
#  5:       
############################################################################
# 1.     ( )   
#     :
goofcat(as.vector(map.lda.c), as.vector(map.rf.c))

# 2.      lda  svm    

# 3.    (CI)
cellStats(map.lda.p1, mean) # 
cellStats(map.lda.p2, mean) # 
cellStats(map.lda.p3, mean) # 
cellStats(map.lda.p4, mean) # 
# CI = 1 - (map.lda.p3 - map.lda.p2)
plot(CI, col = grey(seq(0, 1, len = 25)))

# 4.	        
mod.rf  = randomForest(sid ~ ., data=DSM_data, ntree = 300, mtry = 2) 
importance(mod.rf) #    
varImpPlot(mod.rf)

# 5.	            .
###########################################################################################
# END


#library(nnet) 
#library(C50)
#library(svmpath)
install.packages('C50', dependencies = TRUE)
#install.packages("svmpath", dependencies = TRUE)
install.packages('nnet', dependencies = TRUE)